# Generated by Django 1.10.1 on 2016-09-26 18:46
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    replaces = [
        ("health_check_db", "0001_initial"),
    ]

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="TestModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=128)),
            ],
            options={
                "db_table": "health_check_db_testmodel",
            },
        ),
    ]
